unit imMDIBkg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TimMDIBkg = class(TComponent)
  private
    FBitmap: TBitmap;
    FForm: TForm;
    procedure SetBitmap(Value: TBitmap);
    procedure InternalClientProc(var M: TMessage);
  protected
    FClientInstance: TFarProc;
    FPrevClientProc: TFarProc;
    procedure Loaded; override;
    property Form: TForm read FForm;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Bitmap: TBitmap read FBitmap write SetBitmap;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TimMDIBkg]);
end;

{ TimMDIBkg }

constructor TimMDIBkg.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FForm := AOwner as TForm;
  FBitmap := TBitmap.Create;
end;

destructor TimMDIBkg.Destroy;
begin
  SetWindowLong(Form.ClientHandle, GWL_WNDPROC, Integer(FPrevClientProc));
  FreeObjectInstance(FClientInstance);
  FBitmap.Free;
  inherited Destroy;
end;

procedure TimMDIBkg.InternalClientProc(var M: TMessage);
var
  SrcDC, DstDC: hDC;
  R, C, H, W: Word;
begin
  if not FBitmap.Empty then
    case M.Msg of
      WM_HSCROLL, WM_VSCROLL:
        InvalidateRect(Form.ClientHandle, nil, False);
      WM_ERASEBKGND:
        begin
          SrcDc := Bitmap.Canvas.Handle;
          DstDc := TWMEraseBkGnd(M).DC;
          H := Bitmap.Height;
          W := Bitmap.Width;
          for R := 0 to Form.ClientHeight div H do
            for C := 0 to Form.ClientWidth div W do
              BitBlt(DstDC, C * W, R * H, W, H, SrcDC, 0, 0, SRCCOPY);
          M.Result := 1;
          Exit;
        end;
    end;
  M.Result := CallWindowProc(FPrevClientProc, Form.ClientHandle,
    M.Msg, M.wParam, M.lParam);
end;

procedure TimMDIBkg.Loaded;
begin
  inherited Loaded;
  FClientInstance := MakeObjectInstance(InternalClientProc);
  FPrevClientProc := Pointer(SetWindowLong(
    Form.ClientHandle, GWL_WNDPROC, Integer(FClientInstance)));
end;

procedure TimMDIBkg.SetBitmap(Value: TBitmap);
begin
  FBitmap.Assign(Value);
  FForm.Invalidate;
end;

end.
